<?php
/**
 * ownCloud - Documents App
 *
 * @author Victor Dubiniuk
 * @copyright 2014 Victor Dubiniuk victor.dubiniuk@gmail.com
 *
 * This file is licensed under the Affero General Public License version 3 or
 * later.
 */

namespace OCA\Documents\Controller;

use \OCP\AppFramework\Controller;
use \OCP\IRequest;
use \OCP\AppFramework\Http\JSONResponse;

use \OCA\Documents\Db;
use \OCA\Documents\File;
use \OCA\Documents\Helper;
use OCA\Documents\Filter;

class SessionController extends Controller{
	
	protected $uid;

	public function __construct($appName, IRequest $request, $uid){
		parent::__construct($appName, $request);
		$this->uid = $uid;
	}
	
	/**
	 * @NoAdminRequired
	 * @PublicPage
	 */
	public function joinAsGuest($token, $name){
		$uid = substr($name, 0, 16);
		
		try {
			$file = File::getByShareToken($token);
			if ($file->isPasswordProtected() && !$file->checkPassword('')){
				throw new \Exception('Not authorized');
			}

			$response = array_merge( 
				Db\Session::start($uid, $file),
				array('status'=>'success')
			);
		} catch (\Exception $e){
			Helper::warnLog('Starting a session failed. Reason: ' . $e->getMessage());
			$response = array (
				'status'=>'error'
			);
		}
		
		return $response;
	}
	
	/**
	 * @NoAdminRequired
	 */
	public function joinAsUser($fileId){
		try {
			$view = \OC\Files\Filesystem::getView();
			$path = $view->getPath($fileId);
			
			if ($view->isUpdatable($path)) {
				$file = new File($fileId);
				$response = Db\Session::start($this->uid, $file);
			} else {
				$info = $view->getFileInfo($path);
				$response = array(
					'permissions' => $info['permissions'],
					'id' => $fileId
				);
			}
			$response = array_merge( 
					$response,
					array('status'=>'success')
			);
		} catch (\Exception $e){
			Helper::warnLog('Starting a session failed. Reason: ' . $e->getMessage());
			$response = array (
				'status'=>'error'
			);
		}
		
		return $response;
	}
	
	/**
	 * @NoAdminRequired
	 * @PublicPage
	 * Store the document content to its origin
	 */
	public function save(){
		try {
			$esId = $this->request->server['HTTP_WEBODF_SESSION_ID'];
			if (!$esId){
				throw new \Exception('Session id can not be empty');
			}
			
			$memberId = $this->request->server['HTTP_WEBODF_MEMBER_ID'];
			$currentMember = new Db\Member();
			$currentMember->load($memberId);
			
			//check if member belongs to the session
			if ($esId != $currentMember->getEsId()){
				throw new \Exception($memberId . ' does not belong to session ' . $esId);
			}
			
			// Extra info for future usage
			// $sessionRevision = $this->request->server['HTTP_WEBODF_SESSION_REVISION'];
			
			//NB ouch! New document content is passed as an input stream content
			$stream = fopen('php://input','r');
			if (!$stream){
				throw new \Exception('New content missing');
			}
			$content = stream_get_contents($stream);

			$session = new Db\Session();
			$session->load($esId);
			
			if (!$session->getEsId()){
				throw new \Exception('Session does not exist');
			}

			try {
				if ($currentMember->getIsGuest()){
					$file = File::getByShareToken($currentMember->getToken());
				} else {
					$file = new File($session->getFileId());
				}
				
				list($view, $path) = $file->getOwnerViewAndPath(true);
			} catch (\Exception $e){
				//File was deleted or unshared. We need to save content as new file anyway
				//Sorry, but for guests it would be lost :(
				if ($this->uid){
					$view = new \OC\Files\View('/' . $this->uid . '/files');
		
					$dir = \OCP\Config::getUserValue($this->uid, 'documents', 'save_path', '');
					$path = Helper::getNewFileName($view, $dir . 'New Document.odt');
				} else {
					throw $e;
				}
			}
			
			$member = new Db\Member();
			$members = $member->getActiveCollection($esId);
			$memberIds = array_map(
				function($x){
					return ($x['member_id']);
				},
				$members
			);
			
			// Active users except current user
			$memberCount = count($memberIds) - 1;
			
			if ($view->file_exists($path)){
				$proxyStatus = \OC_FileProxy::$enabled;
				\OC_FileProxy::$enabled = false;	
				$currentHash = sha1($view->file_get_contents($path));
				\OC_FileProxy::$enabled = $proxyStatus;
				
				if (!Helper::isVersionsEnabled() && $currentHash !== $session->getGenesisHash()){
					// Original file was modified externally. Save to a new one
					$path = Helper::getNewFileName($view, $path, '-conflict');
				}
				
				$mimetype = $view->getMimeType($path);
			} else {
				$mimetype = Storage::MIMETYPE_LIBREOFFICE_WORDPROCESSOR;
			}
			
			$data = Filter::write($content, $mimetype);
			
			if ($view->file_put_contents($path, $data['content'])){
				// Not a last user
				if ($memberCount>0){
					// Update genesis hash to prevent conflicts
					Helper::debugLog('Update hash');
					$session->updateGenesisHash($esId, sha1($data['content']));
				} else {
					// Last user. Kill session data
					Db\Session::cleanUp($esId);
				}
				
				$view->touch($path);
			}
			$response = array('status'=>'success');
		} catch (\Exception $e){
			Helper::warnLog('Saving failed. Reason:' . $e->getMessage());
			\OC_Response::setStatus(500);
			$response = array();
		}

		return $response;
	}
}
